@@ -6,7 +6,10 @@
 #include <pthread.h>
 #include <sys/time.h>
 #include <sys/resource.h>
-#include <semaphore.h>
+#include <errno.h>
+#include <stdlib.h>
+#include <stdio.h>
+#include <unistd.h>
 
 #include "list.h"
 #include "md5.h"
@@ -94,18 +97,54 @@ struct verify_header {
 struct group_run_stats {
 	unsigned long long max_run[2], min_run[2];
 	unsigned long long max_bw[2], min_bw[2];
-	unsigned long long io_mb[2];
+	unsigned long long io_kb[2];
 	unsigned long long agg[2];
 };
 
+enum fio_ddir {
+	DDIR_READ = 0,
+	DDIR_WRITE,
+};
+
+/*
+ * What type of allocation to use for io buffers
+ */
+enum fio_memtype {
+	MEM_MALLOC = 0,	/* ordinary malloc */
+	MEM_SHM,	/* use shared memory segments */
+	MEM_MMAP,	/* use anonynomous mmap */
+};
+
+/*
+ * The type of object we are working on
+ */
+enum fio_filetype {
+	FIO_TYPE_FILE = 1,
+	FIO_TYPE_BD,
+	FIO_TYPE_CHAR,
+};
+
+enum fio_iotype {
+	FIO_SYNCIO	= 1 << 0,
+	FIO_MMAPIO	= 1 << 1 | FIO_SYNCIO,
+	FIO_LIBAIO	= 1 << 2,
+	FIO_POSIXAIO	= 1 << 3,
+	FIO_SGIO	= 1 << 4,
+	FIO_SPLICEIO	= 1 << 5 | FIO_SYNCIO,
+};
+
+/*
+ * This describes a single thread/process executing a fio job.
+ */
 struct thread_data {
-	char file_name[256];
-	char directory[256];
+	char name[32];
+	char *file_name;
+	char *directory;
 	char verror[80];
 	pthread_t thread;
 	int thread_number;
 	int groupid;
-	int filetype;
+	enum fio_filetype filetype;
 	int error;
 	int fd;
 	void *mmap;
@@ -114,42 +153,45 @@ struct thread_data {
 	size_t orig_buffer_size;
 	volatile int terminate;
 	volatile int runstate;
-	volatile int old_runstate;
-	unsigned int ddir;
+	enum fio_ddir ddir;
 	unsigned int iomix;
 	unsigned int ioprio;
-	unsigned int sequential;
+
+	unsigned char sequential;
+	unsigned char odirect;
+	unsigned char create_file;
+	unsigned char invalidate_cache;
+	unsigned char create_serialize;
+	unsigned char create_fsync;
+	unsigned char end_fsync;
+	unsigned char sync_io;
+	unsigned char verify;
+	unsigned char use_thread;
+	unsigned char do_disk_util;
+	unsigned char override_sync;
+	unsigned char rand_repeatable;
+
 	unsigned int bs;
 	unsigned int min_bs;
 	unsigned int max_bs;
-	unsigned int odirect;
 	unsigned int thinktime;
 	unsigned int fsync_blocks;
 	unsigned int start_delay;
-	unsigned int timeout;
-	unsigned int io_engine;
-	unsigned int create_file;
+	unsigned long timeout;
+	enum fio_iotype io_engine;
 	unsigned int overwrite;
-	unsigned int invalidate_cache;
 	unsigned int bw_avg_time;
-	unsigned int create_serialize;
-	unsigned int create_fsync;
-	unsigned int end_fsync;
 	unsigned int loops;
 	unsigned long long file_size;
 	unsigned long long real_file_size;
 	unsigned long long file_offset;
 	unsigned long long zone_size;
 	unsigned long long zone_skip;
-	unsigned int sync_io;
-	unsigned int mem_type;
-	unsigned int verify;
+	enum fio_memtype mem_type;
 	unsigned int stonewall;
 	unsigned int numjobs;
-	unsigned int use_thread;
 	unsigned int iodepth;
 	os_cpu_mask_t cpumask;
-	unsigned int jobnum;
 	unsigned int iolog;
 	unsigned int read_iolog;
 	unsigned int write_iolog;
@@ -157,15 +199,22 @@ struct thread_data {
 	unsigned int rwmixread;
 	unsigned int nice;
 
-	char iolog_file[256];
+	char *iolog_file;
 	void *iolog_buf;
 	FILE *iolog_f;
 
-	struct drand48_data bsrange_state;
-	struct drand48_data verify_state;
+	char *sysfs_root;
+	char *ioscheduler;
+
+	os_random_state_t bsrange_state;
+	os_random_state_t verify_state;
 
 	int shm_id;
 
+	/*
+	 * IO engine hooks, contains everything needed to submit an io_u
+	 * to any of the available IO engines.
+	 */
 	void *io_data;
 	char io_engine_name[16];
 	int (*io_prep)(struct thread_data *, struct io_u *);
@@ -176,10 +225,16 @@ struct thread_data {
 	void (*io_cleanup)(struct thread_data *);
 	int (*io_sync)(struct thread_data *);
 
+	/*
+	 * Current IO depth and list of free and busy io_u's.
+	 */
 	unsigned int cur_depth;
 	struct list_head io_u_freelist;
 	struct list_head io_u_busylist;
 
+	/*
+	 * Rate state
+	 */
 	unsigned int rate;
 	unsigned int ratemin;
 	unsigned int ratecycle;
@@ -197,9 +252,12 @@ struct thread_data {
 	unsigned long long zone_bytes;
 	unsigned long long this_io_bytes[2];
 	unsigned long long last_pos;
-	sem_t mutex;
+	volatile int mutex;
 
-	struct drand48_data random_state;
+	/*
+	 * State for random io, a bitmap of blocks done vs not done
+	 */
+	os_random_state_t random_state;
 	unsigned long *file_map;
 	unsigned int num_maps;
 
@@ -220,19 +278,31 @@ struct thread_data {
 	struct timeval start;	/* start of this loop */
 	struct timeval epoch;	/* time job was started */
 
+	/*
+	 * fio system usage accounting
+	 */
 	struct rusage ru_start;
 	struct rusage ru_end;
 	unsigned long usr_time;
 	unsigned long sys_time;
 	unsigned long ctx;
 
-	unsigned int do_disk_util;
-	unsigned int override_sync;
-
-	struct drand48_data rwmix_state;
+	/*
+	 * read/write mixed workload state
+	 */
+	os_random_state_t rwmix_state;
 	struct timeval rwmix_switch;
-	int rwmix_ddir;
+	enum fio_ddir rwmix_ddir;
 
+	/*
+	 * Pre-run and post-run shell
+	 */
+	char *exec_prerun;
+	char *exec_postrun;
+
+	/*
+	 * IO historic logs
+	 */
 	struct list_head io_hist_list;
 	struct list_head io_log_list;
 };
@@ -244,10 +314,6 @@ struct thread_data {
 		snprintf(td->verror, sizeof(td->verror) - 1, "file:%s:%d, error=%s", __FILE__, __LINE__, strerror(e));	\
 	} while (0)
 
-extern int parse_jobs_ini(char *);
-extern int parse_options(int, char **);
-extern void finish_log(struct thread_data *, struct io_log *, const char *);
-extern int init_random_state(struct thread_data *);
 extern struct io_u *__get_io_u(struct thread_data *);
 extern void put_io_u(struct thread_data *, struct io_u *);
 
@@ -258,41 +324,12 @@ extern int exitall_on_terminate;
 extern int thread_number;
 extern int shm_id;
 extern int groupid;
+extern int terse_output;
+extern FILE *f_out;
+extern FILE *f_err;
 
 extern struct thread_data *threads;
 
-enum {
-	DDIR_READ = 0,
-	DDIR_WRITE,
-};
-
-/*
- * What type of allocation to use for io buffers
- */
-enum {
-	MEM_MALLOC,	/* ordinary malloc */
-	MEM_SHM,	/* use shared memory segments */
-	MEM_MMAP,	/* use anonynomous mmap */
-};
-
-/*
- * The type of object we are working on
- */
-enum {
-	FIO_TYPE_FILE = 1,
-	FIO_TYPE_BD,
-	FIO_TYPE_CHAR,
-};
-
-enum {
-	FIO_SYNCIO	= 1 << 0,
-	FIO_MMAPIO	= 1 << 1 | FIO_SYNCIO,
-	FIO_LIBAIO	= 1 << 2,
-	FIO_POSIXAIO	= 1 << 3,
-	FIO_SGIO	= 1 << 4,
-	FIO_SPLICEIO	= 1 << 5 | FIO_SYNCIO,
-};
-
 #define td_read(td)		((td)->ddir == DDIR_READ)
 #define td_write(td)		((td)->ddir == DDIR_WRITE)
 #define td_rw(td)		((td)->iomix != 0)
@@ -340,4 +377,78 @@ struct io_completion_data {
 #define min(a, b)	((a) < (b) ? (a) : (b))
 #endif
 
+/*
+ * Log exports
+ */
+extern int read_iolog_get(struct thread_data *, struct io_u *);
+extern void write_iolog_put(struct thread_data *, struct io_u *);
+extern int init_iolog(struct thread_data *td);
+extern void log_io_piece(struct thread_data *, struct io_u *);
+extern void prune_io_piece_log(struct thread_data *);
+extern void write_iolog_close(struct thread_data *);
+
+/*
+ * Logging
+ */
+extern void add_clat_sample(struct thread_data *, int, unsigned long);
+extern void add_slat_sample(struct thread_data *, int, unsigned long);
+extern void add_bw_sample(struct thread_data *, int);
+extern void show_run_stats(void);
+extern void init_disk_util(struct thread_data *);
+extern void update_rusage_stat(struct thread_data *);
+extern void update_io_ticks(void);
+extern void disk_util_timer_arm(void);
+extern void setup_log(struct io_log **);
+extern void finish_log(struct thread_data *, struct io_log *, const char *);
+extern int setup_rate(struct thread_data *);
+
+/*
+ * Time functions
+ */
+extern unsigned long utime_since(struct timeval *, struct timeval *);
+extern unsigned long mtime_since(struct timeval *, struct timeval *);
+extern unsigned long mtime_since_now(struct timeval *);
+extern unsigned long time_since_now(struct timeval *);
+extern void usec_sleep(struct thread_data *, unsigned long);
+extern void rate_throttle(struct thread_data *, unsigned long, unsigned int);
+
+/*
+ * Init functions
+ */
+extern int parse_options(int, char **);
+extern int init_random_state(struct thread_data *);
+
+/*
+ * This is a pretty crappy semaphore implementation, but with the use that fio
+ * has (just signalling start/go conditions), it doesn't have to be better.
+ * Naturally this would not work for any type of contended semaphore or
+ * for real locking.
+ */
+static inline void fio_sem_init(volatile int volatile *sem, int val)
+{
+	*sem = val;
+}
+
+static inline void fio_sem_down(volatile int volatile *sem)
+{
+	while (*sem == 0)
+		usleep(10000);
+
+	(*sem)--;
+}
+
+static inline void fio_sem_up(volatile int volatile *sem)
+{
+	(*sem)++;
+}
+
+/*
+ * If logging output to a file, stderr should go to both stderr and f_err
+ */
+#define log_err(args...)	do {		\
+	fprintf(f_err, ##args);			\
+	if (f_err != stderr)			\
+		fprintf(stderr, ##args);	\
+	} while (0)
+
 #endif
